package cn.ibizlab.humanresource.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.base.domain.HRLegal;
import cn.ibizlab.humanresource.core.base.filter.HRLegalSearchContext;
import cn.ibizlab.humanresource.core.base.service.IHRLegalService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.base.mapper.HRLegalMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[法人] 服务对象接口实现
 */
@Slf4j
@Service("HRLegalServiceImpl")
public class HRLegalServiceImpl extends ServiceImpl<HRLegalMapper, HRLegal> implements IHRLegalService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HRLegal et) {
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getLegalid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HRLegal> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HRLegal et) {
        hrorganizationService.update(hrlegalInheritMapping.toHrorganization(et));
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("legalid",et.getLegalid())))
            return false;
        CachedBeanCopier.copy(get(et.getLegalid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HRLegal> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        hrorganizationService.remove(key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HRLegal get(String key) {
        HRLegal et = getById(key);
        if(et==null){
            et=new HRLegal();
            et.setLegalid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HRLegal getDraft(HRLegal et) {
        return et;
    }

    @Override
    public boolean checkKey(HRLegal et) {
        return (!ObjectUtils.isEmpty(et.getLegalid()))&&(!Objects.isNull(this.getById(et.getLegalid())));
    }
    @Override
    @Transactional
    public boolean save(HRLegal et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HRLegal et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HRLegal> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HRLegal> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HRLegal> searchDefault(HRLegalSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRLegal> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRLegal>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }






    @Autowired
    cn.ibizlab.humanresource.core.base.mapping.HRLegalInheritMapping hrlegalInheritMapping;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrganizationService hrorganizationService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(HRLegal et){
        if(ObjectUtils.isEmpty(et.getLegalid()))
            et.setLegalid((String)et.getDefaultKey(true));
        cn.ibizlab.humanresource.core.base.domain.HROrganization hrorganization =hrlegalInheritMapping.toHrorganization(et);
        hrorganization.set("organizationtype","LEGAL");
        hrorganizationService.create(hrorganization);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HRLegal> getHrlegalByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HRLegal> getHrlegalByEntities(List<HRLegal> entities) {
        List ids =new ArrayList();
        for(HRLegal entity : entities){
            Serializable id=entity.getLegalid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



