package cn.ibizlab.humanresource.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.base.domain.HROMHierarchyCat;
import cn.ibizlab.humanresource.core.base.filter.HROMHierarchyCatSearchContext;
import cn.ibizlab.humanresource.core.base.service.IHROMHierarchyCatService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.base.mapper.HROMHierarchyCatMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[结构层次类别] 服务对象接口实现
 */
@Slf4j
@Service("HROMHierarchyCatServiceImpl")
public class HROMHierarchyCatServiceImpl extends ServiceImpl<HROMHierarchyCatMapper, HROMHierarchyCat> implements IHROMHierarchyCatService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROMHierarchyService hromhierarchyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROMHierarchyPurposeRefService hromhierarchypurposerefService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HROMHierarchyCat et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getOmhierarchycatid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HROMHierarchyCat> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HROMHierarchyCat et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("omhierarchycatid",et.getOmhierarchycatid())))
            return false;
        CachedBeanCopier.copy(get(et.getOmhierarchycatid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HROMHierarchyCat> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HROMHierarchyCat get(String key) {
        HROMHierarchyCat et = getById(key);
        if(et==null){
            et=new HROMHierarchyCat();
            et.setOmhierarchycatid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HROMHierarchyCat getDraft(HROMHierarchyCat et) {
        return et;
    }

    @Override
    public boolean checkKey(HROMHierarchyCat et) {
        return (!ObjectUtils.isEmpty(et.getOmhierarchycatid()))&&(!Objects.isNull(this.getById(et.getOmhierarchycatid())));
    }
    @Override
    @Transactional
    public boolean save(HROMHierarchyCat et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HROMHierarchyCat et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HROMHierarchyCat> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HROMHierarchyCat> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HROMHierarchyCat> searchDefault(HROMHierarchyCatSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HROMHierarchyCat> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HROMHierarchyCat>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HROMHierarchyCat> getHromhierarchycatByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HROMHierarchyCat> getHromhierarchycatByEntities(List<HROMHierarchyCat> entities) {
        List ids =new ArrayList();
        for(HROMHierarchyCat entity : entities){
            Serializable id=entity.getOmhierarchycatid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



