package cn.ibizlab.humanresource.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.base.domain.HROperationUnit;
import cn.ibizlab.humanresource.core.base.filter.HROperationUnitSearchContext;
import cn.ibizlab.humanresource.core.base.service.IHROperationUnitService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.base.mapper.HROperationUnitMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[运营单位] 服务对象接口实现
 */
@Slf4j
@Service("HROperationUnitServiceImpl")
public class HROperationUnitServiceImpl extends ServiceImpl<HROperationUnitMapper, HROperationUnit> implements IHROperationUnitService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HROperationUnit et) {
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getOperationunitid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HROperationUnit> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HROperationUnit et) {
        hrorganizationService.update(hroperationunitInheritMapping.toHrorganization(et));
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("operationunitid",et.getOperationunitid())))
            return false;
        CachedBeanCopier.copy(get(et.getOperationunitid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HROperationUnit> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        hrorganizationService.remove(key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HROperationUnit get(String key) {
        HROperationUnit et = getById(key);
        if(et==null){
            et=new HROperationUnit();
            et.setOperationunitid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HROperationUnit getDraft(HROperationUnit et) {
        return et;
    }

    @Override
    public boolean checkKey(HROperationUnit et) {
        return (!ObjectUtils.isEmpty(et.getOperationunitid()))&&(!Objects.isNull(this.getById(et.getOperationunitid())));
    }
    @Override
    @Transactional
    public boolean save(HROperationUnit et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HROperationUnit et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HROperationUnit> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HROperationUnit> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HROperationUnit> searchDefault(HROperationUnitSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HROperationUnit> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HROperationUnit>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }






    @Autowired
    cn.ibizlab.humanresource.core.base.mapping.HROperationUnitInheritMapping hroperationunitInheritMapping;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrganizationService hrorganizationService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(HROperationUnit et){
        if(ObjectUtils.isEmpty(et.getOperationunitid()))
            et.setOperationunitid((String)et.getDefaultKey(true));
        cn.ibizlab.humanresource.core.base.domain.HROrganization hrorganization =hroperationunitInheritMapping.toHrorganization(et);
        hrorganization.set("organizationtype","OPERATIONUNIT");
        hrorganizationService.create(hrorganization);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HROperationUnit> getHroperationunitByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HROperationUnit> getHroperationunitByEntities(List<HROperationUnit> entities) {
        List ids =new ArrayList();
        for(HROperationUnit entity : entities){
            Serializable id=entity.getOperationunitid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



