package cn.ibizlab.humanresource.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.base.domain.HROrgContact;
import cn.ibizlab.humanresource.core.base.filter.HROrgContactSearchContext;
import cn.ibizlab.humanresource.core.base.service.IHROrgContactService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.base.mapper.HROrgContactMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[组织联系信息] 服务对象接口实现
 */
@Slf4j
@Service("HROrgContactServiceImpl")
public class HROrgContactServiceImpl extends ServiceImpl<HROrgContactMapper, HROrgContact> implements IHROrgContactService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrganizationService hrorganizationService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HROrgContact et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getHrorgcontactid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HROrgContact> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HROrgContact et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("hrorgcontactid",et.getHrorgcontactid())))
            return false;
        CachedBeanCopier.copy(get(et.getHrorgcontactid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HROrgContact> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HROrgContact get(String key) {
        HROrgContact et = getById(key);
        if(et==null){
            et=new HROrgContact();
            et.setHrorgcontactid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HROrgContact getDraft(HROrgContact et) {
        return et;
    }

    @Override
    public boolean checkKey(HROrgContact et) {
        return (!ObjectUtils.isEmpty(et.getHrorgcontactid()))&&(!Objects.isNull(this.getById(et.getHrorgcontactid())));
    }
    @Override
    @Transactional
    public boolean save(HROrgContact et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HROrgContact et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HROrgContact> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HROrgContact> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HROrgContact> selectByHrorganizationid(String organizationid) {
        return baseMapper.selectByHrorganizationid(organizationid);
    }

    @Override
    public void removeByHrorganizationid(String organizationid) {
        this.remove(new QueryWrapper<HROrgContact>().eq("hrorganizationid",organizationid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HROrgContact> searchDefault(HROrgContactSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HROrgContact> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HROrgContact>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HROrgContact> getHrorgcontactByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HROrgContact> getHrorgcontactByEntities(List<HROrgContact> entities) {
        List ids =new ArrayList();
        for(HROrgContact entity : entities){
            Serializable id=entity.getHrorgcontactid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



