package cn.ibizlab.humanresource.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.base.domain.HROrganization;
import cn.ibizlab.humanresource.core.base.filter.HROrganizationSearchContext;
import cn.ibizlab.humanresource.core.base.service.IHROrganizationService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.base.mapper.HROrganizationMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[组织] 服务对象接口实现
 */
@Slf4j
@Service("HROrganizationServiceImpl")
public class HROrganizationServiceImpl extends ServiceImpl<HROrganizationMapper, HROrganization> implements IHROrganizationService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHREmployeeService hremployeeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrgAddressService hrorgaddressService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrgContactService hrorgcontactService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHRPostService hrpostService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROMHierarchyService hromhierarchyService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HROrganization et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getOrganizationid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HROrganization> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HROrganization et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("organizationid",et.getOrganizationid())))
            return false;
        CachedBeanCopier.copy(get(et.getOrganizationid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HROrganization> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HROrganization get(String key) {
        HROrganization et = getById(key);
        if(et==null){
            et=new HROrganization();
            et.setOrganizationid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HROrganization getDraft(HROrganization et) {
        return et;
    }

    @Override
    public boolean checkKey(HROrganization et) {
        return (!ObjectUtils.isEmpty(et.getOrganizationid()))&&(!Objects.isNull(this.getById(et.getOrganizationid())));
    }
    @Override
    @Transactional
    public boolean save(HROrganization et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HROrganization et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HROrganization> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HROrganization> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HROrganization> searchDefault(HROrganizationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HROrganization> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HROrganization>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HROrganization> getHrorganizationByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HROrganization> getHrorganizationByEntities(List<HROrganization> entities) {
        List ids =new ArrayList();
        for(HROrganization entity : entities){
            Serializable id=entity.getOrganizationid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



