package cn.ibizlab.humanresource.core.humanresource.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.humanresource.domain.HRDistribution;
import cn.ibizlab.humanresource.core.humanresource.filter.HRDistributionSearchContext;
import cn.ibizlab.humanresource.core.humanresource.service.IHRDistributionService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.humanresource.mapper.HRDistributionMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[工作人员分配] 服务对象接口实现
 */
@Slf4j
@Service("HRDistributionServiceImpl")
public class HRDistributionServiceImpl extends ServiceImpl<HRDistributionMapper, HRDistribution> implements IHRDistributionService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHREmployeeService hremployeeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHRPostService hrpostService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HRDistribution et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getHrdistributionid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HRDistribution> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HRDistribution et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("hrdistributionid",et.getHrdistributionid())))
            return false;
        CachedBeanCopier.copy(get(et.getHrdistributionid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HRDistribution> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HRDistribution get(String key) {
        HRDistribution et = getById(key);
        if(et==null){
            et=new HRDistribution();
            et.setHrdistributionid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HRDistribution getDraft(HRDistribution et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(HRDistribution et) {
        return (!ObjectUtils.isEmpty(et.getHrdistributionid()))&&(!Objects.isNull(this.getById(et.getHrdistributionid())));
    }
    @Override
    @Transactional
    public boolean save(HRDistribution et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HRDistribution et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HRDistribution> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HRDistribution> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HRDistribution> selectByHremployeeid(String employeeid) {
        return baseMapper.selectByHremployeeid(employeeid);
    }

    @Override
    public void removeByHremployeeid(String employeeid) {
        this.remove(new QueryWrapper<HRDistribution>().eq("hremployeeid",employeeid));
    }

	@Override
    public List<HRDistribution> selectByHrpostid(String hrpostid) {
        return baseMapper.selectByHrpostid(hrpostid);
    }

    @Override
    public void removeByHrpostid(String hrpostid) {
        this.remove(new QueryWrapper<HRDistribution>().eq("hrpostid",hrpostid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HRDistribution> searchDefault(HRDistributionSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRDistribution> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRDistribution>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(HRDistribution et){
        //实体关系[DER1N_HRDISTRIBUTION_HREMPLOYEE_HREMPLOYEEID_327A4D]
        if(!ObjectUtils.isEmpty(et.getHremployeeid())){
            cn.ibizlab.humanresource.core.humanresource.domain.HREmployee hremployee=et.getHremployee();
            if(ObjectUtils.isEmpty(hremployee)){
                cn.ibizlab.humanresource.core.humanresource.domain.HREmployee majorEntity=hremployeeService.get(et.getHremployeeid());
                et.setHremployee(majorEntity);
                hremployee=majorEntity;
            }
            et.setHremployeename(hremployee.getEmployeename());
        }
        //实体关系[DER1N_HRDISTRIBUTION_HRPOST_HRPOSTID]
        if(!ObjectUtils.isEmpty(et.getHrpostid())){
            cn.ibizlab.humanresource.core.humanresource.domain.HRPost hrpost=et.getHrpost();
            if(ObjectUtils.isEmpty(hrpost)){
                cn.ibizlab.humanresource.core.humanresource.domain.HRPost majorEntity=hrpostService.get(et.getHrpostid());
                et.setHrpost(majorEntity);
                hrpost=majorEntity;
            }
            et.setHrpostname(hrpost.getHrpostname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HRDistribution> getHrdistributionByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HRDistribution> getHrdistributionByEntities(List<HRDistribution> entities) {
        List ids =new ArrayList();
        for(HRDistribution entity : entities){
            Serializable id=entity.getHrdistributionid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



