package cn.ibizlab.humanresource.core.humanresource.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.humanresource.domain.HRDutyEdu;
import cn.ibizlab.humanresource.core.humanresource.filter.HRDutyEduSearchContext;
import cn.ibizlab.humanresource.core.humanresource.service.IHRDutyEduService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.humanresource.mapper.HRDutyEduMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[职务教育] 服务对象接口实现
 */
@Slf4j
@Service("HRDutyEduServiceImpl")
public class HRDutyEduServiceImpl extends ServiceImpl<HRDutyEduMapper, HRDutyEdu> implements IHRDutyEduService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHRDutyService hrdutyService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HRDutyEdu et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getHrdutyeduid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HRDutyEdu> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HRDutyEdu et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("hrdutyeduid",et.getHrdutyeduid())))
            return false;
        CachedBeanCopier.copy(get(et.getHrdutyeduid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HRDutyEdu> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HRDutyEdu get(String key) {
        HRDutyEdu et = getById(key);
        if(et==null){
            et=new HRDutyEdu();
            et.setHrdutyeduid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HRDutyEdu getDraft(HRDutyEdu et) {
        return et;
    }

    @Override
    public boolean checkKey(HRDutyEdu et) {
        return (!ObjectUtils.isEmpty(et.getHrdutyeduid()))&&(!Objects.isNull(this.getById(et.getHrdutyeduid())));
    }
    @Override
    @Transactional
    public boolean save(HRDutyEdu et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HRDutyEdu et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HRDutyEdu> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HRDutyEdu> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HRDutyEdu> selectByHrdutyid(String hrdutyid) {
        return baseMapper.selectByHrdutyid(hrdutyid);
    }

    @Override
    public void removeByHrdutyid(String hrdutyid) {
        this.remove(new QueryWrapper<HRDutyEdu>().eq("hrdutyid",hrdutyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HRDutyEdu> searchDefault(HRDutyEduSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRDutyEdu> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRDutyEdu>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HRDutyEdu> getHrdutyeduByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HRDutyEdu> getHrdutyeduByEntities(List<HRDutyEdu> entities) {
        List ids =new ArrayList();
        for(HRDutyEdu entity : entities){
            Serializable id=entity.getHrdutyeduid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



