package cn.ibizlab.humanresource.core.humanresource.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.humanresource.domain.HREmpState;
import cn.ibizlab.humanresource.core.humanresource.filter.HREmpStateSearchContext;
import cn.ibizlab.humanresource.core.humanresource.service.IHREmpStateService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.humanresource.mapper.HREmpStateMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[员工状态] 服务对象接口实现
 */
@Slf4j
@Service("HREmpStateServiceImpl")
public class HREmpStateServiceImpl extends ServiceImpl<HREmpStateMapper, HREmpState> implements IHREmpStateService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHREmployeeService hremployeeService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HREmpState et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getHrempstateid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HREmpState> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HREmpState et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("hrempstateid",et.getHrempstateid())))
            return false;
        CachedBeanCopier.copy(get(et.getHrempstateid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HREmpState> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HREmpState get(String key) {
        HREmpState et = getById(key);
        if(et==null){
            et=new HREmpState();
            et.setHrempstateid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HREmpState getDraft(HREmpState et) {
        return et;
    }

    @Override
    public boolean checkKey(HREmpState et) {
        return (!ObjectUtils.isEmpty(et.getHrempstateid()))&&(!Objects.isNull(this.getById(et.getHrempstateid())));
    }
    @Override
    @Transactional
    public boolean save(HREmpState et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HREmpState et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HREmpState> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HREmpState> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HREmpState> searchDefault(HREmpStateSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HREmpState> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HREmpState>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HREmpState> getHrempstateByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HREmpState> getHrempstateByEntities(List<HREmpState> entities) {
        List ids =new ArrayList();
        for(HREmpState entity : entities){
            Serializable id=entity.getHrempstateid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



