package cn.ibizlab.humanresource.core.humanresource.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.humanresource.domain.HRPostRel;
import cn.ibizlab.humanresource.core.humanresource.filter.HRPostRelSearchContext;
import cn.ibizlab.humanresource.core.humanresource.service.IHRPostRelService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.humanresource.mapper.HRPostRelMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[职位关系] 服务对象接口实现
 */
@Slf4j
@Service("HRPostRelServiceImpl")
public class HRPostRelServiceImpl extends ServiceImpl<HRPostRelMapper, HRPostRel> implements IHRPostRelService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHRPostService hrpostService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HRPostRel et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getHrpostrelid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HRPostRel> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HRPostRel et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("hrpostrelid",et.getHrpostrelid())))
            return false;
        CachedBeanCopier.copy(get(et.getHrpostrelid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HRPostRel> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HRPostRel get(String key) {
        HRPostRel et = getById(key);
        if(et==null){
            et=new HRPostRel();
            et.setHrpostrelid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HRPostRel getDraft(HRPostRel et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(HRPostRel et) {
        return (!ObjectUtils.isEmpty(et.getHrpostrelid()))&&(!Objects.isNull(this.getById(et.getHrpostrelid())));
    }
    @Override
    @Transactional
    public boolean save(HRPostRel et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HRPostRel et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HRPostRel> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HRPostRel> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HRPostRel> selectByHrpostid(String hrpostid) {
        return baseMapper.selectByHrpostid(hrpostid);
    }

    @Override
    public void removeByHrpostid(String hrpostid) {
        this.remove(new QueryWrapper<HRPostRel>().eq("hrpostid",hrpostid));
    }

	@Override
    public List<HRPostRel> selectByPhrpostid(String hrpostid) {
        return baseMapper.selectByPhrpostid(hrpostid);
    }

    @Override
    public void removeByPhrpostid(String hrpostid) {
        this.remove(new QueryWrapper<HRPostRel>().eq("phrpostid",hrpostid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HRPostRel> searchDefault(HRPostRelSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRPostRel> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRPostRel>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(HRPostRel et){
        //实体关系[DER1N_HRPOSTREL_HRPOST_HRPOSTID]
        if(!ObjectUtils.isEmpty(et.getHrpostid())){
            cn.ibizlab.humanresource.core.humanresource.domain.HRPost hrpost=et.getHrpost();
            if(ObjectUtils.isEmpty(hrpost)){
                cn.ibizlab.humanresource.core.humanresource.domain.HRPost majorEntity=hrpostService.get(et.getHrpostid());
                et.setHrpost(majorEntity);
                hrpost=majorEntity;
            }
            et.setHrpostname(hrpost.getHrpostname());
        }
        //实体关系[DER1N_HRPOSTREL_HRPOST_PHRPOSTID]
        if(!ObjectUtils.isEmpty(et.getPhrpostid())){
            cn.ibizlab.humanresource.core.humanresource.domain.HRPost phrpost=et.getPhrpost();
            if(ObjectUtils.isEmpty(phrpost)){
                cn.ibizlab.humanresource.core.humanresource.domain.HRPost majorEntity=hrpostService.get(et.getPhrpostid());
                et.setPhrpost(majorEntity);
                phrpost=majorEntity;
            }
            et.setPhrpostname(phrpost.getHrpostname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HRPostRel> getHrpostrelByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HRPostRel> getHrpostrelByEntities(List<HRPostRel> entities) {
        List ids =new ArrayList();
        for(HRPostRel entity : entities){
            Serializable id=entity.getHrpostrelid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



