package cn.ibizlab.humanresource.core.humanresource.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.humanresource.domain.HRPost;
import cn.ibizlab.humanresource.core.humanresource.filter.HRPostSearchContext;
import cn.ibizlab.humanresource.core.humanresource.service.IHRPostService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.humanresource.mapper.HRPostMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[职位] 服务对象接口实现
 */
@Slf4j
@Service("HRPostServiceImpl")
public class HRPostServiceImpl extends ServiceImpl<HRPostMapper, HRPost> implements IHRPostService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHRDistributionService hrdistributionService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHRPostRelService hrpostrelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHRDutyService hrdutyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrganizationService hrorganizationService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HRPost et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getHrpostid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HRPost> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HRPost et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("hrpostid",et.getHrpostid())))
            return false;
        CachedBeanCopier.copy(get(et.getHrpostid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HRPost> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HRPost get(String key) {
        HRPost et = getById(key);
        if(et==null){
            et=new HRPost();
            et.setHrpostid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HRPost getDraft(HRPost et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(HRPost et) {
        return (!ObjectUtils.isEmpty(et.getHrpostid()))&&(!Objects.isNull(this.getById(et.getHrpostid())));
    }
    @Override
    @Transactional
    public boolean save(HRPost et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HRPost et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HRPost> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HRPost> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HRPost> selectByHrdutyid(String hrdutyid) {
        return baseMapper.selectByHrdutyid(hrdutyid);
    }

    @Override
    public void removeByHrdutyid(String hrdutyid) {
        this.remove(new QueryWrapper<HRPost>().eq("hrdutyid",hrdutyid));
    }

	@Override
    public List<HRPost> selectByHrorganizationid(String organizationid) {
        return baseMapper.selectByHrorganizationid(organizationid);
    }

    @Override
    public void removeByHrorganizationid(String organizationid) {
        this.remove(new QueryWrapper<HRPost>().eq("hrorganizationid",organizationid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HRPost> searchDefault(HRPostSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRPost> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRPost>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(HRPost et){
        //实体关系[DER1N_HRPOST_HRDUTY_HRDUTYID]
        if(!ObjectUtils.isEmpty(et.getHrdutyid())){
            cn.ibizlab.humanresource.core.humanresource.domain.HRDuty hrduty=et.getHrduty();
            if(ObjectUtils.isEmpty(hrduty)){
                cn.ibizlab.humanresource.core.humanresource.domain.HRDuty majorEntity=hrdutyService.get(et.getHrdutyid());
                et.setHrduty(majorEntity);
                hrduty=majorEntity;
            }
            et.setHrdutyname(hrduty.getHrdutyname());
        }
        //实体关系[DER1N_HRPOST_HRORGANIZATION_HRORGANIZATIONID_6BD9DC]
        if(!ObjectUtils.isEmpty(et.getHrorganizationid())){
            cn.ibizlab.humanresource.core.base.domain.HROrganization hrorganization=et.getHrorganization();
            if(ObjectUtils.isEmpty(hrorganization)){
                cn.ibizlab.humanresource.core.base.domain.HROrganization majorEntity=hrorganizationService.get(et.getHrorganizationid());
                et.setHrorganization(majorEntity);
                hrorganization=majorEntity;
            }
            et.setHrorganizationname(hrorganization.getOrganizationname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HRPost> getHrpostByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HRPost> getHrpostByEntities(List<HRPost> entities) {
        List ids =new ArrayList();
        for(HRPost entity : entities){
            Serializable id=entity.getHrpostid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



